package de.dhsn.oop.ui;

import de.dhsn.oop.data.TodoList;
import de.dhsn.oop.ui.helpers.TodoListListModel;

import javax.swing.*;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.List;
import java.util.Objects;

public class MainWindow extends JFrame implements ActionListener, ListSelectionListener {
    private JButton addNewList;
    private List<TodoList> lists;
    private JList<String> list;

    /**
     * Main window constructor, takes a (potentially empty) list of TodoList
     * this list of lists is supposed to keep all application state for easy file save and open operations
     * @param lists
     * @throws HeadlessException
     */
    public MainWindow(List<TodoList> lists) throws HeadlessException {
        super();
        this.lists = lists;
        Container cp = getContentPane();
        cp.removeAll();
        setTitle("Todo List");
        setSize(new Dimension(800, 600));
        setMinimumSize(new Dimension(400, 250));
        setLayout(new BorderLayout());

        list = new JList<>();
        list.setModel(new TodoListListModel(lists));
        list.addListSelectionListener(this);
        cp.add(list, BorderLayout.NORTH);

        addNewList = new JButton("Neue Liste Anlegen");
        addNewList.addActionListener(this);
        cp.add(addNewList, BorderLayout.SOUTH);

        pack();
        setVisible(true);

        this.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                System.out.println("exit test");
                System.out.println(lists);
                System.exit(0);
            }
        });
    }

    /**
     * listener function for the add new list button on the main page
     * @param e the event to be processed
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        new NewListDialog(this);
    }

    /**
     * callback function to be called with a title for a new list
     * this is called from NewListDialog when the user presses the save button
     * @param title
     * @return
     */
    public void nldCallback(String title){
        boolean result = lists.add(new TodoList(title));

        //overwrite list contents
        list.setModel(new TodoListListModel(lists));
    }

    /**
     * handle click events on the list of list titles
     * displays the list in a new window
     * @param e the event that characterizes the change.
     */
    @Override
    public void valueChanged(ListSelectionEvent e) {
        // two events are emitted - only execute the code once
        if (e.getValueIsAdjusting()){
            //show new window for the selected list
            new TodoListView(lists.get(list.getSelectedIndex()));
        }
    }
}
